==> pam-0.3.tar.gz: Linux-PAM-0.3/libpam/pam_start.c <==
/* pam_start.c */

/* Creator Marc Ewing
 *
 * $Id: pam_start.c,v 1.3 1996/03/16 22:37:52 morgan Exp $
 *
 * $Log: pam_start.c,v $
 * Revision 1.3  1996/03/16 22:37:52  morgan
 * made all of the input arguments to pam_start constants
 *
 * Revision 1.2  1996/03/16 22:05:37  morgan
 * added RCS
 * changed service to be const char *
 *
 */

#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <syslog.h>

#include "pam_private.h"

int pam_start (
    const char *service_name,
    const char *user,
    const struct pam_conv *pam_conversation,
    pam_handle_t **pamh)
{
    if ((*pamh = malloc(sizeof(**pamh))) == NULL) {
	_pam_log_error("pam_start: malloc failed for *pamh");
	return (PAM_BUF_ERR);
    }

    if (service_name) {
	if (((*pamh)->service_name = strdup(service_name)) == NULL) {
	    _pam_log_error("pam_start: strdup failed for service name");
	    DROP(*pamh);
	    return (PAM_BUF_ERR);
	}
    } else
       	(*pamh)->service_name = NULL;

    if (user) {
	if (((*pamh)->user = strdup(user)) == NULL) {
	    _pam_log_error("pam_start: strdup failed for user");
	    DROP((*pamh)->service_name);
	    DROP(*pamh);
	    return (PAM_BUF_ERR);
	}
    } else
	(*pamh)->user = NULL;

    (*pamh)->tty = NULL;
    (*pamh)->tuser = NULL;       /* temp user name for pam_get_user() */
    (*pamh)->prompt = NULL;      /* temp user name for pam_get_user() */
    (*pamh)->ruser = NULL;
    (*pamh)->rhost = NULL;
    (*pamh)->authtok = NULL;
    (*pamh)->oldauthtok = NULL;

    /* added by AGM 1996/2/5 -- malloc some space for the pam_conv struct */

    if (pam_conversation == NULL
	|| ((*pamh)->pam_conversation = (struct pam_conv *)
	    malloc(sizeof(struct pam_conv))) == NULL) {
	_pam_log_error("pam_start: malloc failed for pam_conv");
	DROP((*pamh)->service_name);
	DROP((*pamh)->user);
	DROP(*pamh);
	return (PAM_BUF_ERR);
    } else {
	memcpy((*pamh)->pam_conversation
	       ,pam_conversation
	       ,sizeof(struct pam_conv));
    }

    (*pamh)->data = NULL;

    _pam_start_handlers(*pamh);

    /* According to the SunOS man pages, loading modules and resolving
     * symbols happens on the first call from the application.
     * _pam_init_handlers(*pamh);
     */
    
    return PAM_SUCCESS;
}

==> pam-0.21.tar.gz: Linux-PAM-0.21/libpam/pam_start.c <==
/* pam_start.c */

#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <syslog.h>

#include "pam_private.h"

int pam_start (
    char *service_name,
    char *user,
    struct pam_conv *pam_conversation,
    pam_handle_t **pamh)
{
    if ((*pamh = malloc(sizeof(**pamh))) == NULL) {
	_pam_log_error("pam_start: malloc failed for *pamh");
	return (PAM_BUF_ERR);
    }

    if (service_name) {
	if (((*pamh)->service_name = strdup(service_name)) == NULL) {
	    _pam_log_error("pam_start: strdup failed for service name");
	    DROP(*pamh);
	    return (PAM_BUF_ERR);
	}
    } else
       	(*pamh)->service_name = NULL;

    if (user) {
	if (((*pamh)->user = strdup(user)) == NULL) {
	    _pam_log_error("pam_start: strdup failed for user");
	    DROP((*pamh)->service_name);
	    DROP(*pamh);
	    return (PAM_BUF_ERR);
	}
    } else
	(*pamh)->user = NULL;

    (*pamh)->tty = NULL;
    (*pamh)->tuser = NULL;       /* temp user name for pam_get_user() */
    (*pamh)->prompt = NULL;      /* temp user name for pam_get_user() */
    (*pamh)->ruser = NULL;
    (*pamh)->rhost = NULL;
    (*pamh)->authtok = NULL;
    (*pamh)->oldauthtok = NULL;

    /* added by AGM 1996/2/5 -- malloc some space for the pam_conv struct */

    if (pam_conversation == NULL
	|| ((*pamh)->pam_conversation = (struct pam_conv *)
	    malloc(sizeof(struct pam_conv))) == NULL) {
	_pam_log_error("pam_start: malloc failed for pam_conv");
	DROP((*pamh)->service_name);
	DROP((*pamh)->user);
	DROP(*pamh);
	return (PAM_BUF_ERR);
    } else {
	memcpy((*pamh)->pam_conversation
	       ,pam_conversation
	       ,sizeof(struct pam_conv));
    }

    (*pamh)->data = NULL;

    _pam_start_handlers(*pamh);

    /* According to the SunOS man pages, loading modules and resolving
     * symbols happens on the first call from the application.
     * _pam_init_handlers(*pamh);
     */
    
    return PAM_SUCCESS;
}
